@@ -9,7 +9,7 @@ test_that("attribute topology basics", {
   y <- sf::st_transform(y, 5070)
 
   # generate a network from the geometry
-  z <- make_attribute_topology(hy(y), 10)
+  z <- make_attribute_topology(y, 10)
 
   expect_equal(sum(z$toid == 0), 1)
 
@@ -19,7 +19,7 @@ test_that("attribute topology basics", {
   # make sure that all are the same from scratch
   expect_true(all(sapply(unique(x$id), function(i) {
     xtid <- order(x[x$id == i,]$toid)
-    ztid <- order(z[z$id == i,]$toid)
+    ztid <- order(z[z$COMID == i,]$toid)
 
     all(xtid == ztid)
   })))
@@ -34,5 +34,5 @@ test_that("attribute topology basics", {
   a <- add_toids(z, return_dendritic = FALSE)
 
   # make sure we get the same toids!
-  all(z$toid == a$toid)
+  expect_true(all(z$toid == a$toid))
 })
