@@ -123,6 +123,13 @@ test_that("main", {
 
   upmain <- distinct(select(x, id, upmain))
 
+  y <- make_index_ids(x[,c("id", "toid")])
+
+  z <- make_fromids(y, return_list = TRUE)
+
+  expect_error(navigate_network_dfs(z, 8897784, "upmain"), "include main element")
+  expect_error(navigate_network_dfs(y, 8897784, "downmain"), "must include main element")
+
   y <- make_index_ids(x)
 
   z <- make_fromids(y, return_list = TRUE, upmain = upmain)
